/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
}

var sameSize = function (xMatrix) {
    var restMatrix = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        restMatrix[_i - 1] = arguments[_i];
    }
    var _a = getSize(xMatrix), rows = _a.rows, cols = _a.cols;
    return !restMatrix.some(function (matrix) {
        var size = getSize(matrix);
        return size.cols !== cols || size.rows !== rows;
    });
};
var validMatrix = function () {
    var input = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        input[_i] = arguments[_i];
    }
    return !input.some(function (item) { return !(Array.isArray(item) && Array.isArray(item[0])); });
};

var ops = {
    add: function (n1, n2) { return n1 + n2; },
    subtract: function (n1, n2) { return n1 - n2; },
    divide: function (n1, n2) { return n1 / n2; },
    multiply: function (n1, n2) { return n1 * n2; }
};
var getSize = function (matrix) {
    return { cols: matrix[0].length, rows: matrix.length };
};
var computeMatrices = function (operation, xInput, yInput) {
    if (!sameSize(xInput, yInput)) {
        throw new Error('All ndarrays must have the same size and dimensions');
    }
    return xInput.map(function (row, rowIndex) {
        return row.map(function (cell, cellIndex) { return (ops[operation](cell, yInput[rowIndex][cellIndex])); });
    });
};
var computeScalarAndMatrix = function (operation, num, matrix) {
    return matrix.map(function (row) {
        return row.map(function (cell) { return (ops[operation](num, cell)); });
    });
};
var compute = function (operation) {
    var inputs = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        inputs[_i - 1] = arguments[_i];
    }
    if (inputs.length === 0) {
        return [];
    }
    if (inputs.length === 1) {
        return inputs[0];
    }
    return inputs.reduce(function (xInput, yInput) {
        if (validMatrix(xInput, yInput)) {
            return computeMatrices(operation, xInput, yInput);
        }
        var isXInputNumber = typeof xInput === 'number';
        var isYInputNumber = typeof yInput === 'number';
        if (isXInputNumber && isYInputNumber) {
            return ops[operation](xInput, yInput);
        }
        return computeScalarAndMatrix(operation, (isXInputNumber ? xInput : yInput), (isYInputNumber ? xInput : yInput));
    });
};

var add = function () {
    var inputs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        inputs[_i] = arguments[_i];
    }
    return compute.apply(void 0, __spreadArray(['add'], inputs, false));
};

var shape = function (matrix) {
    return getSize(matrix);
};

var dotMatrices = function (xMatrix, yMatrix) {
    if (!validMatrix(xMatrix, yMatrix)) {
        throw new Error('Invalid input');
    }
    var rows = shape(xMatrix).rows;
    var cols = shape(yMatrix).cols;
    return new Array(rows).fill(undefined).map(function (row, i) {
        if (row === void 0) { row = new Array(cols).fill(undefined); }
        return row.map(function (_, j) {
            return xMatrix[0].reduce(function (sum, _, k) { return (sum + xMatrix[i][k] * yMatrix[k][j]); }, 0);
        });
    });
};
var dot = function () {
    var input = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        input[_i] = arguments[_i];
    }
    return input.reduce(dotMatrices);
};

var exponent = function (input) {
    if (!Array.isArray(input)) {
        return Math.exp(input);
    }
    return input.map(function (row) { return row.map(function (cell) { return Math.exp(cell); }); });
};

var multiply = function () {
    var input = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        input[_i] = arguments[_i];
    }
    return compute.apply(void 0, __spreadArray(['multiply'], input, false));
};

var negative = function (input) {
    return multiply(-1, input);
};

var random = function (rows, cols) {
    return Array(rows).fill([]).map((function (_) { return Array(cols).fill(0).map(function (__) { return Math.random(); }); }));
};

var reciprocal = function (input) {
    if (typeof input === 'number') {
        return 1 / input;
    }
    return input.map(function (row) { return row.map(function (cell) { return 1 / cell; }); });
};

var subtract = function () {
    var inputs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        inputs[_i] = arguments[_i];
    }
    return compute.apply(void 0, __spreadArray(['subtract'], inputs, false));
};

var transpose = function (matrix) {
    return matrix[0].map(function (_, cellIndex) { return matrix.map(function (row) { return row[cellIndex]; }); });
};

var sigmoid = function (matrix) {
    return reciprocal(add(1, exponent(negative(matrix))));
};

var derivative = function (matrix) {
    return multiply(matrix, subtract(1, matrix));
};

var train = function (input, output, options) {
    var transposedOutput = transpose(output);
    var cols = shape(input).cols;
    var initialSynapse = (options.initialNetwork && options.initialNetwork.synapses[0]) || options.initialSynapse || subtract(multiply(2, random(cols, 1)));
    var synapses = [initialSynapse];
    var layers = [];
    var errors = [];
    var deltas = [];
    for (var i = 0; i < options.iterations; i++) {
        layers[0] = input;
        layers[1] = sigmoid(dot(layers[0], synapses[0]));
        errors[1] = subtract(transposedOutput, layers[1]);
        deltas[1] = multiply(errors[1], derivative(layers[1]));
        synapses[0] = add(synapses[0], dot(transpose(layers[0]), deltas[1]));
    }
    return { layers: layers, synapses: synapses, deltas: deltas, errors: errors };
};
var run = function (input, trainData) {
    return sigmoid(dot(input, trainData.synapses[0]))[0][0];
};

export { run, train };
//# sourceMappingURL=index.es.js.map
